/* ColormapMaker v1.00, freeware. By Paul Siramy. YYYY/MM/DD : 2010/05/14 */

#include <stdio.h>
#include <allegro.h>

#define SUCCESS 0
#define FAIL    1

#define COLORMAPMAKER_VERSION 1.00


int main(int argc, char * argv [])
{
 FILE   * in         = NULL;
 FILE   * bin        = NULL;
 FILE   * out        = NULL;
 char   buffer [900] = "";
 int    c            = 0;
 BITMAP * bmp        = NULL;
 int    row          = 0;
 char   * extension  = NULL;
 int    x            = 0;
 int    y            = 0;
 char   * cmapfile   = NULL;
 int    param_out    = 0;
 int    param_quiet  = 0;


 if (install_allegro(SYSTEM_AUTODETECT, & errno, atexit) != 0)
  {
   allegro_message("ERROR. Can't install the Allegro library.\n");
   return FAIL;
  }

 sprintf(buffer, "%s v%0.2f", get_filename(argv[0]), COLORMAPMAKER_VERSION);
 set_window_title(buffer);
 set_color_depth(8);
 set_color_conversion(COLORCONV_NONE);

 if (argc < 2)
  {
   allegro_message("ERROR. You must provide a INI file as the 1st parameter in the command line.\n");
   return FAIL;
  }

 in = fopen(argv[1], "rt");
 if (in == NULL)
  {
   allegro_message("ERROR. Can't open file \"%s\".\n", argv[1]);
   return FAIL;
  }

 if (argc == 3)
  {
   if (stricmp(argv[2], "/Q") == 0)
    param_quiet = 2;
   else
    param_out = 2;
  }
 else if (argc == 4)
  {
   if (stricmp(argv[2], "/Q") == 0)
    {
     param_quiet = 2;
     param_out = 3;
    }
   else if (stricmp(argv[3], "/Q") == 0)
    {
     param_quiet = 3;
     param_out = 2;
    }
  }

 if (param_out == 0)
  cmapfile = "colormap.bin";
 else
  cmapfile = argv[param_out];

 out = fopen(cmapfile, "wb");
 if (out == NULL)
  {
   allegro_message("ERROR. Can't create file \"%s\".\n", cmapfile);
   return FAIL;
  }

 while (fgets(buffer, sizeof(buffer), in) == buffer)
  {
   row++;

   for (c = 0; c < strlen(buffer); c++)
    {
     if ((buffer[c] == 0x0D) || (buffer[c] == 0x0A))
      buffer[c] = 0x00;
    }

   if (strlen(buffer) == 0)
    continue;

   if (buffer[0] == '#')
    continue;

   if (strlen(buffer) < 5)
    {
     fclose(in);
     fclose(out);
     allegro_message("ERROR. Row %d in file \"%s\" is less than 5 characters.\n", row, argv[1]);
     return FAIL;
    }

   for (c = strlen(buffer) - 1; (buffer[c] != '.') && (c >= 0); c--)
    {}

   if (c < 0)
    {
     fclose(in);
     fclose(out);
     allegro_message("ERROR. Row %d in file \"%s\" has no extension (no '.' character found)\n", row, argv[1]);
     return FAIL;
    }

   extension = buffer + c;
   if ((stricmp(extension, ".BMP") == 0) || (stricmp(extension, ".LBM") == 0) || (stricmp(extension, ".PCX") == 0) || (stricmp(extension, ".TGA") == 0))
    {
     bmp = load_bitmap(buffer, NULL);
     if (bmp == NULL)
      {
       fclose(in);
       fclose(out);
       allegro_message("ERROR. Can't open the image \"%s\", probably bad file format.\n", buffer);
       return FAIL;
      }

     if ((bmp->w * bmp->h) != 256)
      {
       fclose(in);
       fclose(out);
       allegro_message("ERROR. The image \"%s\" is %d * %d pixels (%d pixels in total). It should be an image of exactly 256 pixels in total.\n", buffer, bmp->w, bmp->h, (bmp->w * bmp->h));
       destroy_bitmap(bmp);
       return FAIL;
      }

     if (bitmap_color_depth(bmp) != 8)
      {
       fclose(in);
       fclose(out);
       allegro_message("ERROR. The image \"%s\" is in %d bits-per-pixel mode (%d colors). It should be in 8 bpp mode (256 colors).\n", buffer, bitmap_color_depth(bmp), (1 << bitmap_color_depth(bmp)));
       destroy_bitmap(bmp);
       return FAIL;
      }

     for (y = 0; y < bmp->h; y++)
      {
       for (x = 0; x < bmp->w; x++)
        {
         c = getpixel(bmp, x, y);
         if ((c < 0) || (c > 255))
          {
           fclose(in);
           fclose(out);
           allegro_message("ERROR. For some unknown reason the pixel [%d, %d] in the image \"%s\" is color %d, while it should be between 0 and 255.\n", x, y, buffer, c);
           destroy_bitmap(bmp);
           return FAIL;
          }
         fputc(c, out);
        }
      }
     destroy_bitmap(bmp);
    }
   else if (stricmp(extension, ".BIN") == 0)
    {
     bin = fopen(buffer, "rb");

     if (bin == NULL)
      {
       fclose(in);
       fclose(out);
       allegro_message("ERROR. Can't open the binary file \"%s\"\n", buffer);
       return FAIL;
      }

     c = fgetc(bin);
     while (c != EOF)
      {
       fputc(c, out);
       c = fgetc(bin);
      }

     fclose(bin);
    }
   else
    {
     fclose(in);
     fclose(out);
     allegro_message("ERROR. Unknown file extension \"%s\" at row %d in file \"%s\"\n", extension, row, argv[1]);
     return FAIL;
    }
  }

 fclose(in);
 fclose(out);

 if (param_quiet == 0)
  allegro_message("DONE. File \"%s\" sucessfully made based upon file \"%s\"\n", cmapfile, argv[1]);

 return SUCCESS;
}
END_OF_MAIN()
